unit simcom;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls,
	Forms, Dialogs,
	CPort, Menus,
	registry, stdctrls, extctrls;

const
	cpbaud = br19200;
	cpdata = dbeight;
	cppari = prnone;
	cppchk = false;

type
	Tcom =
	class(TDataModule)
    ComPort1: TComPort;
		procedure 	DataModuleCreate(Sender: TObject);
		procedure 	DataModuleDestroy(Sender: TObject);
	private		{ Private-Deklarationen }
		frec:			boolean;
		cp:			Tcomport;
		rcvstr:		string[255];
		procedure 	PortClick(Sender: TObject);
		procedure 	cpError(Sender: TObject; Errors: TComErrors);
		procedure 	cpRxChar(Sender: TObject; Count: Integer);
	public		{ Public-Deklarationen }
		ports:		tstringlist;
		mon:			TMemo;
		hex:			boolean;
		procedure	SndByte(dat: word);
		function		RcvByte: byte;
		procedure	addports(menuitem: tmenuitem);
		property		Rec:		boolean		read frec;
	end;

var
	com: 			Tcom;

implementation

uses simu;

{$R *.DFM}

// eingabe byte
function	Tcom.RcvByte: byte;
begin
	result := 0;
	if frec
	then begin
		result := byte(rcvstr[1]);
		delete(rcvstr, 1, 1);
	end;
	frec := (length(rcvstr) > 0);
end;

// ausgabe byte
procedure Tcom.SndByte(dat: word);
var
	v:			string;
	c:			char;
begin
	if (dat >= $20)
	and (dat <= $80)
	then c := char(dat)
	else c := '.';
	v := '>';
	if assigned(cp)
	then begin
		v := '> not open';
		if cp.Connected
		then begin
			cp.writestr(char(dat));
			v := '>';
		end;
	end;
	if assigned(mon)
	then begin
		mon.Lines.Add(format('%3.3x %s %s', [dat, c, v]));
	end;
end;

procedure Tcom.cpRxChar(Sender: TObject; Count: Integer);
var
	s,
	o:		string;
	i:			integer;
	w:			byte;
begin
	with sender as tcomport
	do begin
		Readstr(s, Count);
//		rcvstr := rcvstr + str;
		i := 1;
		o := '';
		while i <= count
		do begin
//			wrk.sfrs[_SBUF] := w;
//			wrk.bitm(_RI, true);
//			w := byte(str[i]);
			o := o + format('\%2.2x', [byte(s[i])]);
			inc(i);
		end;
		simulator.mseri.lines.Add(o);
		simulator.sndl := simulator.sndl + length(o) + 2;
		simulator.startserin;
	end;
end;

procedure Tcom.cpError(Sender: TObject; Errors: TComErrors);
begin
	if not assigned(mon)
	then exit;
	case byte(errors) of
	byte(ceFrame):		mon.lines.add('***** Frame Error');
	byte(ceRxParity): mon.lines.add('***** Rx Parity Error');
	byte(ceOverrun):  mon.lines.add('***** Overrun');
	byte(ceBreak):    mon.lines.add('***** Break');
	byte(ceIO):       mon.lines.add('***** IO Error');
	byte(ceMode):     mon.lines.add('***** Mode Error');
	byte(ceRxOver):   mon.lines.add('***** Rx Over');
	byte(ceTxFull):   mon.lines.add('***** Tx Full');
	end;
end;

procedure Tcom.DataModuleCreate(Sender: TObject);
var
	i:			integer;
	s:			string;
	reg:		treginifile; 
begin
	inherited;
	ports := tstringlist.create;
	reg := TReginifile.Create;
	reg.RootKey := HKEY_LOCAL_MACHINE;
	reg.OpenKey('\HARDWARE\DEVICEMAP\SERIALCOMM', false);
	reg.GetValueNames(ports);
	reg.OpenKey('\HARDWARE\DEVICEMAP', false);
	for i := 0 to ports.Count - 1
	do begin
		s := ports[i];
		ports[i] := reg.ReadString('SERIALCOMM', s, '');
	end;
	ports.Sort;
	reg.free;
	cp := tcomport.Create(self);
	cp.OnError := cperror;
	cp.OnRxChar := cprxchar;
	cp.BaudRate := cpbaud;
	cp.DataBits := cpdata;
	cp.Parity.Bits := cppari;
	cp.Parity.Check := cppchk;
	cp.StopBits := sbOneStopBit;
end;

procedure Tcom.addports(menuitem: tmenuitem);
var
	i:			integer;
	m:			tmenuitem;
begin

	m := tmenuitem.Create(self);
	m.Caption := 'kein Port';
	m.RadioItem := true;
	m.checked := true;
	m.GroupIndex := 1;
	m.Name := menuitem.Name + '00';
	m.Tag := 0;
	m.OnClick := portclick;
	menuitem.Add(m);
	for i := 0 to com.ports.Count - 1
	do begin
		m := tmenuitem.Create(self);
		m.Caption := com.ports[i];
		m.RadioItem := true;
		m.GroupIndex := 1;
		m.Tag := i + 1;
		m.Name := menuitem.Name + '_' + m.Caption;
		m.OnClick := portclick;
		menuitem.Add(m);
	end;
end;

procedure Tcom.PortClick(Sender: TObject);
var
	mi:		tmenuitem;
begin
	mi := tmenuitem(sender);
	cp.Close;
	mi.parent.checked := false;
	mi.Parent.Items[0].Checked := true;
	mi.checked := true;
	if mi.menuindex > 0
	then begin
		cp.Port := mi.Caption;
		cp.Open;
		if cp.Connected
		then begin
			mi.Checked := true;
			mi.parent.checked := true;
		end;
	end;
end;

procedure Tcom.DataModuleDestroy(Sender: TObject);
begin
	if cp.Connected
	then cp.Close;
	cp.Free;
end;

end.
